@@ -1,15 +1,18 @@
 use std::env;
 use std::fs;
+use std::io;
 use std::path::{Path, PathBuf};
 use std::process::Command;
 
 fn main() {
     let https = env::var("CARGO_FEATURE_HTTPS").is_ok();
     let ssh = env::var("CARGO_FEATURE_SSH").is_ok();
+    let zlib_ng_compat = env::var("CARGO_FEATURE_ZLIB_NG_COMPAT").is_ok();
 
-    if env::var("LIBGIT2_SYS_USE_PKG_CONFIG").is_ok() {
+    // To use zlib-ng in zlib-compat mode, we have to build libgit2 ourselves.
+    if !zlib_ng_compat {
         let mut cfg = pkg_config::Config::new();
-        if let Ok(lib) = cfg.atleast_version("0.28.0").probe("libgit2") {
+        if let Ok(lib) = cfg.atleast_version("1.0.0").probe("libgit2") {
             for include in &lib.include_paths {
                 println!("cargo:root={}", include.display());
             }
@@ -19,7 +22,7 @@ fn main() {
 
     if !Path::new("libgit2/.git").exists() {
         let _ = Command::new("git")
-            .args(&["submodule", "update", "--init"])
+            .args(&["submodule", "update", "--init", "libgit2"])
             .status();
     }
 
@@ -197,8 +200,12 @@ fn cp_r(from: impl AsRef<Path>, to: impl AsRef<Path>) {
 }
 
 fn add_c_files(build: &mut cc::Build, path: impl AsRef<Path>) {
-    for e in path.as_ref().read_dir().unwrap() {
-        let e = e.unwrap();
+    // sort the C files to ensure a deterministic build for reproducible builds
+    let dir = path.as_ref().read_dir().unwrap();
+    let mut paths = dir.collect::<io::Result<Vec<_>>>().unwrap();
+    paths.sort_by_key(|e| e.path());
+
+    for e in paths {
         let path = e.path();
         if e.file_type().unwrap().is_dir() {
             // skip dirs for now
